
// test example:
//  - dynamic alocation of square matrix, void* result
//  - various access methods

#include <stdio.h>      // printf()
#include <stdlib.h>     // calloc(), free()

// allocation example: create initialized test matrix
inline void *AllocMat(size_t n)
{
    double (*p)[n][n] = malloc(n * n * sizeof(double));
    if (p)
        for (int y = 0; y < n; ++y)
            for (int x = 0; x < n; ++x)
                (*p)[y][x] = 10.0*y+x;
    return p;
}

// inline does not work if compiled without optimization, we need
// non-inline external function definition:
extern void *AllocMat(size_t n);       // define external version (C99)

// simple test
int main(void)
{
    const int N = 10;

    void *v = AllocMat(N);             // allocation, initialization

    if(!v)
        return 1;

    // there is no bounds checking
    int x=1;
    int y=2;
    printf("\n x=%d, y=%d \n\n", x, y);

    printf("using pointer to matrix:                  ");
    double (*m)[N][N] = v;
    printf("  (*m)[y][x]==%g\n", (*m)[y][x]);    // exact

    printf("using pointer to first row of matrix:     ");
    double (*mm)[N] = v;
    printf("  mm[y][x]==%g\n", mm[y][x]);        // like function parameter

    printf("using pointer to first element of matrix: ");
    double *p = v;
    printf("  p[y*N + x]==%g\n", p[y*N + x]);    // do not use - too low level
    printf("\n\n");

    free(m);    // deallocation
    return 0;
}

