/* cos-tab.c
 *
 * Program tiskne tabulku funkčních hodnot funkce cos(x) získanou
 * výpočtem řady a standardní funkcí cos() s přesností double.
 *
 * Sledujte nepřesnost výpočtu součtů a vliv počtu členů řady
 *
 * UPOZORNĚNÍ:
 *  - nejde o efektivní řešení
 *  - program sestavte s matematickou knihovnou (-lm)
 *
 * Počet prvků řady lze zadat jako argument programu (implicitně 10).
 *
 */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

int N = 10;			/* implicitní počet prvků řady */

double factorial(int i)
{
    double x = 1.0L;
    while (i > 1)
	x *= i--;
    return x;
}

double my_cos(double x)
{
    /* výpočet prvků řady, sčítání od konce pro vyšší přesnost
     *
     *  cos(x) = 1 - frac{x^2}{2!} + frac{x^4}{4!} - frac{x^6}{6!} + ...
     *
     *  platí pro x v rozsahu -1 .. 1
     *
     */
    double sum = 0;
    for (int i = N - 1; i >= 0; i--)
	sum += ((i & 1) ? -1 : 1) * pow(x, 2 * i) / factorial(2 * i);
    return sum;
}

int main(int argc, char *argv[])
{
    double x;
    if (argc == 2)
	N = atoi(argv[1]);
    if (N < 1 || N > 50)
	return 1;
    printf("%21s | %21s | %s\n", "x", "my_cos", "abs_error");
    for (x = -1.0; x <= 1.0; x += 1.0/64) {
	double y = my_cos(x);
	double y2 = cos(x);
	printf("%21.16f | %21.16f | %g\n", x, y, y2 - y);
    }
}
