
// use gcc -std=c99
// convert memory contents to binary representation in string
// little endian numbers expected, show MSB first, MSbit first
// + demo: dump number representation for int/float/double/long double

#include <stdio.h>
#include <string.h>
#include <assert.h>

void mem2binstr(const void *ptr, unsigned size, char *s)
{
    const char *p = ptr;
    int i;
    int mask;
    if(size==0) return;
    for (i = size; i>0; i--) { // little endian
	int byte = p[i-1];     // 8-bit byte expected
	for (mask = 0x80; mask & 0xFF; mask >>= 1)
	    *s++ = (byte & mask) ? '1' : '0';
	*s++ = ' ';
    }
    *--s = '\0'; // end of string, remove last space
}

// demo
int main()
{
    int data;
    char s[100];
    printf("MSB first\n");
    for (data = -3; data < 10; data += 1) {
	printf("%d = \n", data);
        int i = data;
	mem2binstr(&i, sizeof(i), s);
	printf("%s (int)\n", s);
        float f = data;
	mem2binstr(&f, sizeof(f), s);
	printf("%s (float)\n", s);
        double d = data;
	mem2binstr(&d, sizeof(d), s);
	printf("%s (double)\n", s);
        long double ld = data;
	mem2binstr(&ld, 10/*WARNING*/, s);
	printf("%s (long double)\n", s);
    }
    return 0;
}
