
// Small example how to create consistent enum/array tables
// Warning: this code works only for enums without
//          explicit value assignment: ENUMCONST=5,
// Public domain

#include <stdio.h>

// Function definitions (simple code can be part of the table, too):
void function_empty(void) {}
void function_ID2(void) { puts("test function for ID2"); }
void function_ID3(void) { puts("test function for ID3"); }

#undef DEF      // will be defined later

//////////////////////////////////////////////////////////////////
// Symbol table definition (definitions can be #included from file)
// tuple: (enum constant, extra C string literal, function)
//////////////////////////////////////////////////////////////////
#define TABLE_X                                 \
    DEF(ID0,"ID0...",function_empty)            \
    DEF(ID1,"ID1...",function_empty)            \
    DEF(ID2,"string for ID2",function_ID2)      \
    DEF(ID3,"ID3...",function_ID3)              \
    DEF(ID4,"ID4...",function_empty)            \
    DEF(ID5,"ID5...",function_empty)            \

//////////////////////////////////////////////////////////////////
// End od table definition
//////////////////////////////////////////////////////////////////

// Define enum:
enum E {
#define DEF(a,b,c) a,
    TABLE_X             // first column selected: enum constants
#undef DEF
    MAX_ID // sentinel value == number of items (can be in TABLE_X)
};

// Define string table for enum symbols:
const char * E2str[] = {        // no array size specified
#define DEF(a,b,c) #a,
    TABLE_X             // strings made from first column
#undef DEF
// no need for sentinel here
};

// It is possible to define wrapper function
inline const char *E_to_string(enum E e) {
    if((unsigned)e < MAX_ID) return E2str[e];
    return "Error: bad value of enum E";
}
extern const char *E_to_string(enum E e); // C99 external definition

// Define extra string table for enum symbols:
const char * E_xstr[] = {       // no array size specified
#define DEF(a,b,c) b,
    TABLE_X             // second column
#undef DEF
// no need for sentinel here
};

// Define function pointer table for enum symbols:
void (*E_fun[MAX_ID])(void) = { // array size specified
#define DEF(a,b,c) &c,
    TABLE_X             // third column
#undef DEF
// sentinel here is not allowed
};

// simple test
int main() {
    enum E v = ID2;
    printf("%s\n", E_to_string(v));
    printf("%s\n", E_xstr[v]);
    E_fun[v]();
    v = ID3;
    printf("%s\n", E_to_string(v));
    printf("%s\n", E_xstr[v]);
    E_fun[v]();
    return v;
}
