
// Simple example of C11 threads.h usage
// Tested with GCC8.3+glibc-2.28 on Debian10/buster (2019)
// Tested with GCC12.2+glibc-2.36 on Debian12/bookworm (2024)
// Needs gcc flag -lpthread
// Source: stackoverflow

// TODO: measure time vs sequential

#include <stdio.h>
#include <threads.h>

#ifndef NUM_THREADS
#  define NUM_THREADS   2       // should be number of CPU cores
#endif
#define N               1000000 // maximum depends on ulimit -s
#define TIMES           1000    // how many times to repeat

// user data for thread
struct thread_data_t {
    int thread_number;          // user assigned number
    long sum;                   // thread result
};
typedef struct thread_data_t thread_data_t;

// each thread has one item there, no need for locking:
static thread_data_t threadData[NUM_THREADS];
// "false sharing": do not access frequently

int threadFunction(void *_data)
{
    thread_data_t *data = _data;
    int local[N] = { 1, 2, 3, }; // TODO: random?
    long sum = 0;
    printf("#thread%ld start\n", *data);
    for (int j = 0; j < TIMES; ++j)
        for (int i = 0; i < N; ++i)
            sum += local[i];
    printf("#thread%d end, sum=%ld\n", data->thread_number, sum);
    data->sum = sum;
    return 0;
}

int main(void)
{
    thrd_t tId[NUM_THREADS];

    // init thread data
    for (int i = 0; i < NUM_THREADS; ++i) {
        threadData[i].thread_number = i;
        threadData[i].sum = 0L;
    }

    // start all threads
    for (int i = 0; i < NUM_THREADS; ++i) {
        if (thrd_create(tId + i, threadFunction, threadData + i) != thrd_success) {
            printf("thread%d creation error\n", i);
            return 0;
        }
    }

    // wait until all threads terminate
    for (int i = 0; i < NUM_THREADS; ++i)
        thrd_join(tId[i], NULL);

    // print thread results
    for (int i = 0; i < NUM_THREADS; ++i)
        printf("Thread%d: data=%ld\n", i, threadData[i].sum);

    return 0;
}
